
# Python modules
import sys
import argparse
import os
from _ast import List

# AIM modules
try:
    import f90nml
except:
    print "Unable to import f90nml module - Try \" pip install f90nml \""
    raise SystemExit


def read_fun3d_nml(alt_dir = None):
#############################################################
### Read and evaluate fun3d namelist ########################
#############################################################    
    
    if alt_dir is None:
        alt_dir = 'Description'
  
    if os.path.isfile(alt_dir + '/fun3d.nml'):
        print '\n\tReading fun3d.nml .....'
        # Read namelist
        nml = f90nml.read(alt_dir + '/fun3d.nml', row_major=False)
        return nml

    return (None)

def write_nml(fname,nml):
#############################################################
### Write an arbitary namelist to file ######################
#############################################################
     nml.write(fname,force=True)

##################
###    MAIN    ###
##################
if( __name__ == "__main__"):

    
    #Read the command line
    parser = argparse.ArgumentParser(description = 'FUN3D namelist AIM for CAPS',
                                     prog = 'fun3d_Namelist_AIM',
                                     formatter_class = argparse.ArgumentDefaultsHelpFormatter)
 
    #Give the option to choose what type of case management to use.
    parser.add_argument('-dir'  , nargs=1, type=str,   default='Description', help = 'Directory for fun3d.nml')
    parser.add_argument('-proj' , nargs=1, type=str,   default=None         , help = 'Sets project root name')
    parser.add_argument('-mesh' , nargs=1, type=str,   default=None         , help = 'Sets mesh type and binary (0) / ascii (1) flag')
    parser.add_argument('-mach' , nargs=1, type=float, default=None         , help = 'Sets the Mach number')
    parser.add_argument('-re'   , nargs=1, type=float, default=None         , help = 'Sets the Reynolds number')
    parser.add_argument('-alpha', nargs=1, type=float, default=None         , help = 'Sets the angle of attack')
    parser.add_argument('-beta' , nargs=1, type=float, default=None         , help = 'Sets the angle of yaw (side-slip)')
    
    parser.add_argument('-aref' , nargs=1, type=float, default=None, help = 'Sets reference area')
    parser.add_argument('-xmom' , nargs=1, type=float, default=None, help = 'Sets x-moment center')
    parser.add_argument('-ymom' , nargs=1, type=float, default=None, help = 'Sets y-moment center')
    parser.add_argument('-zmom' , nargs=1, type=float, default=None, help = 'Sets z-moment center')
    parser.add_argument('-force_mom', nargs=1, type=str, default=None, help = 'Sets reference area and x,y,z-moment center')
    
    parser.add_argument('-temperature_bc', nargs=1, type=str, default=None, help = 'Sets temperature boundary conditions - should' + 
                                                                                   'be followed by a space seperated string of' + 
                                                                                   ' boundary IDs and temperature')
    
       
    parser.add_argument('-visc', nargs=1, type=str, default=None,choices ={'inviscid','laminar','turbulent'} , help = 'Sets gov. eqn visous term')
    parser.add_argument('-eqn',  nargs=1, type=str, default=None,choices ={'incompressible','compressible'}  , help = 'Sets gov. eqns')
    
    parser.add_argument('-cfl_schedule', nargs=1, type=str, default=None, help = 'Sets CFL schedule')
    parser.add_argument('-cfl_schedule_iter', nargs=1, type=str, default=None, help = 'Sets CFL schedule interations')
    parser.add_argument('-noninertial_rot_center', nargs=1, type=str, default=None, help = 'Sets noninterial rotation center')
    parser.add_argument('-noninertial_rot_rate', nargs=1, type=str, default=None, help = 'Sets noninterial rotation rate')
    
    parser.add_argument('-iter', nargs=1, type=float, default=None, help = 'Sets number of iterations for simulation')
    
    parser.add_argument('-restart', nargs=1, type=str, default=None, choices={'off','on','on_nohistorykept'}, help = 'Sets read restart option')
    
    parser.add_argument('-dummy', nargs=1, type=str, default=None, help = 'Do nothing')
     
    # Parse inputs
    args = parser.parse_args()
   
    # Get run directory
    alt_dir = str(args.dir[0])
    
    # Load fun3d namelist 
    nml = read_fun3d_nml(alt_dir)
    
    # Create a new namelist if no namelist is found  
    if nml is None:
        nml = f90nml.Namelist()
        print "\tCreating namelist"
    else:    
        print "\tAppending namelist"
          
    # Go through varible options
    if args.proj:
        if 'project' not in nml:
            nml['project'] = f90nml.Namelist()
        
        nml['project']['project_rootname'] = str(args.proj[0])
    
    if args.mesh:
        if 'raw_grid' not in nml:
            nml['raw_grid'] = f90nml.Namelist()
        
        temp = args.mesh[0].rstrip('\n').split(' ')
        
        nml['raw_grid']['grid_format'] = str(temp[0])
        
        if float(temp[1]) == 1:
            nml['raw_grid']['data_format'] = 'ascii'
        else:
            nml['raw_grid']['data_format'] = 'stream'
            
    if args.mach:
        if 'reference_physical_properties' not in nml:
            nml['reference_physical_properties'] = f90nml.Namelist()
            
        nml['reference_physical_properties']['mach_number'] = float(args.mach[0])
        
    if args.re:
        if 'reference_physical_properties' not in nml:
            nml['reference_physical_properties'] = f90nml.Namelist()
            
        nml['reference_physical_properties']['reynolds_number'] = float(args.re[0])
        
    if args.alpha:
        if 'reference_physical_properties' not in nml:
            nml['reference_physical_properties'] = f90nml.Namelist()
            
        nml['reference_physical_properties']['angle_of_attack'] = float(args.alpha[0])
        
    if args.beta:
        if 'reference_physical_properties' not in nml:
            nml['reference_physical_properties'] = f90nml.Namelist()
            
        nml['reference_physical_properties']['angle_of_yaw'] = float(args.beta[0])
    
    if args.force_mom:
        if 'force_moment_integ_properties' not in nml:
            nml['force_moment_integ_properties'] = f90nml.Namelist()
            
        temp = args.force_mom[0].rstrip('\n').split(' ')
        
        if (len(temp) != 4):
            print "\t4 values (Ref area, x-mom, y-mom, and z-mom center) need to be provided in the -force_mom string"
        else:
            
            nml['force_moment_integ_properties']['area_reference']  = float(temp[0])
            nml['force_moment_integ_properties']['x_moment_center'] = float(temp[1])
            nml['force_moment_integ_properties']['y_moment_center'] = float(temp[2])
            nml['force_moment_integ_properties']['z_moment_center'] = float(temp[3])
     
    if args.visc:
        if 'governing_equations' not in nml:
            nml['governing_equations'] = f90nml.Namelist()
        
        nml['governing_equations']['viscous_terms'] = str(args.visc[0])
     
    if args.eqn:
        if 'governing_equations' not in nml:
            nml['governing_equations'] = f90nml.Namelist()
        
        nml['governing_equations']['eqn_type'] = str(args.eqn[0])
    
    if args.cfl_schedule:
        if 'nonlinear_solver_parameters' not in nml:
            nml['nonlinear_solver_parameters'] = f90nml.Namelist()
            
        temp = args.cfl_schedule[0].rstrip('\n').split(' ')
    
        if (len(temp) != 2):
            print "\t2 values (Start and ending) need to be provided in the -cfl_schedule string"
        else:
            
            if 'schedule_cfl' not in nml['nonlinear_solver_parameters']:
                nml['nonlinear_solver_parameters']['schedule_cfl'] = [None]*2
                 
            nml['nonlinear_solver_parameters']['schedule_cfl'][0] = float(temp[0])
            nml['nonlinear_solver_parameters']['schedule_cfl'][1] = float(temp[1])
    
    if args.cfl_schedule_iter:
        if 'nonlinear_solver_parameters' not in nml:
            nml['nonlinear_solver_parameters'] = f90nml.Namelist()
            
        temp = args.cfl_schedule_iter[0].rstrip('\n').split(' ')
        
        if (len(temp) != 2):
            print "\t2 values (Start and ending) need to be provided in the -cfl_schedule_iter string"
        else:
            
            if 'schedule_iteration' not in nml['nonlinear_solver_parameters']:
                nml['nonlinear_solver_parameters']['schedule_iteration'] = [None]*2
                 
            nml['nonlinear_solver_parameters']['schedule_iteration'][0] = int(temp[0])
            nml['nonlinear_solver_parameters']['schedule_iteration'][1] = int(temp[1]) 
    
    if args.noninertial_rot_center:
        if 'noninertial_reference_frame' not in nml:
            nml['noninertial_reference_frame'] = f90nml.Namelist()
        
        temp = args.noninertial_rot_center[0].rstrip('\n').split(' ')
        
        if (len(temp) != 3):
            print "\t3 values (x, y, and z) need to be provided in the -noninertial_rot_center string"
        else:
            nml['noninertial_reference_frame']['noninertial'] = True
            nml['noninertial_reference_frame']['rotation_center_x'] = float(temp[0])
            nml['noninertial_reference_frame']['rotation_center_y'] = float(temp[1]) 
            nml['noninertial_reference_frame']['rotation_center_z'] = float(temp[2]) 
    
    if args.noninertial_rot_rate:
        if 'noninertial_reference_frame' not in nml:
            nml['noninertial_reference_frame'] = f90nml.Namelist()
        
        
        temp = args.noninertial_rot_rate[0].rstrip('\n').split(' ')
        
        if (len(temp) != 3):
            print "\t3 values (x, y, and z) need to be provided in the -noninertial_rot_rate string"
        else:
            nml['noninertial_reference_frame']['noninertial'] = True
        
            nml['noninertial_reference_frame']['rotation_rate_x'] = float(temp[0])
            nml['noninertial_reference_frame']['rotation_rate_y'] = float(temp[1]) 
            nml['noninertial_reference_frame']['rotation_rate_z'] = float(temp[2]) 
    
    if args.iter:
        if 'code_run_control' not in nml:
            nml['code_run_control'] = f90nml.Namelist()

        nml['code_run_control']['steps'] = int(args.iter[0])
    
    if args.restart:
        if 'code_run_control' not in nml:
            nml['code_run_control'] = f90nml.Namelist()
            
        nml['code_run_control']['restart_read'] = str(args.restart[0])
        
    if args.temperature_bc:
        if 'boundary_conditions' not in nml:
            nml['boundary_conditions'] = f90nml.Namelist()
        
        #print "\tTemperature boundary conditions in the fun3d.nml will be overwritten"
        
        # Filter out spaces
        temp = filter(None, args.temperature_bc[0].rstrip('\n').split(' '))
            
        if 'wall_temperature' in nml['boundary_conditions']:
            wallTemp = nml['boundary_conditions']['wall_temperature']
        else: 
            wallTemp = None
            
        if 'wall_temp_flag' in nml['boundary_conditions']:
            wallTempFlag = nml['boundary_conditions']['wall_temp_flag']
        else:
            wallTempFlag = None
            
        # Get max boundary index
        max_index = 0;
        for i in range(0,len(temp),2):
            if int(temp[i]) > max_index:
                max_index = int(temp[i])
            elif int(temp[i]) < 0:
                print "\tNegative boundary index detected - invalid value"
                raise SystemExit 
         
        nml['boundary_conditions']['wall_temperature'] = [None]*max_index
        nml['boundary_conditions']['wall_temp_flag'] = [None]*max_index
        
        #Copy incoming values back in to temperature arrays
        if wallTemp is not None:
            if not isinstance(wallTemp, list):
                wallTemp = [wallTemp]
                
            ii = 0
            for i in wallTemp:
                nml['boundary_conditions']['wall_temperature'][ii] = i
                ii = ii + 1
                
        if wallTempFlag is not None:
            if not isinstance(wallTempFlag, list):
                wallTempFlag = [wallTempFlag]
                
            ii = 0
            for i in wallTempFlag:
                nml['boundary_conditions']['wall_temp_flag'][ii] = i
                ii = ii + 1
            
        # Populate temperature arrays from new inputs
        for i in range(max_index):
            found = False
            for ii in range(0,len(temp),2):
                 
                if int(temp[ii])-1 == i:
                    nml['boundary_conditions']['wall_temperature'][i] = float(temp[ii+1])
                    nml['boundary_conditions']['wall_temp_flag'][i] = True
                    found = True
                    break
                else: 
                    found = False
                     
#             if found == False:
#                 nml.data.boundary_conditions.wall_temperature[i] = 0
#                 nml.data.boundary_conditions.wall_temp_flag[i] = False
            
    # Write new namelist
    if args.dir is None:
        write_nml('Description/' + 'fun3d.nml', nml)
    else:
        write_nml(alt_dir + '/fun3d.nml', nml)
