// Extract flow variables from SU2 surface csv file (connectivity is ignored) - dataMatrix = [numVariable][numDataPoint]
int su2_readAeroLoad(char *filename, int *numVariable, char **variableName[], int *numDataPoint, double ***dataMatrix) {

    int status; // Function return
    int i, j; // Indexing

    size_t linecap = 0;
    int number_of_lines = 0;
    int ch=0;

    char *line = NULL; // Temporary line holder
    char *tempStr = NULL, *tempStr2 = NULL; // Temporary strings
    int stringLen = 0; // Length of string holder
    char comma[1];

    FILE *fp; // File pointer

    // Open file
    fp = fopen(filename, "r");
    if (fp == NULL) {
        printf("Unable to open file: %s\n", filename);
        return CAPS_IOERR;
    }

    printf("Reading FUN3D AeroLoad File - %s!!!!!!\n", filename);
    printf(" Pass 1: to count number of nodes\n");

    // Count the number of lines to determine number of data points
    while(!feof(fp)) {
        ch = fgetc(fp);
        if(ch == '\n') {
            number_of_lines++;
        }
    }   
    fclose(fp);

    number_of_lines -=1;
    printf("Number of data points = %d\n", number_of_lines);

    fp = fopen(filename, "r");
    if (fp == NULL) {
        printf("Unable to open file: %s\n", filename);
        return CAPS_IOERR;
    }

    printf(" Pass 2: filling data matrix\n");

    *numVariable = 0;
    *numDataPoint = 0;
    // Loop through file line by line until we have determined how many variables and data points there are
    while (*numVariable == 0 || *numDataPoint == 0) {

        // Get line from file
        status = getline(&line, &linecap, fp);
        if (status < 0) break;

        // Get variable list if available in file line
        // if (strncmp("variables=", line, strlen("variables=")) == 0) {

            // Pull out substring at first occurrence of "
            tempStr = strstr(line, "\"");

            // Create a temperory string of the variables in a the folling format - ["a","ae"]
            stringLen  = strlen(tempStr)-1 + 2;

            tempStr2 = (char *) EG_alloc((stringLen +1)*sizeof(char *));
            if (tempStr2 == NULL) {
            	fclose(fp);
            	return EGADS_MALLOC;
            }

            tempStr2[0] = '[';
            strncpy(tempStr2+1, tempStr, strlen(tempStr)-1);
            tempStr2[stringLen-1] = ']';
            tempStr2[stringLen] = '\0';

            j=0;
            stringLen = strlen(tempStr2);

            for(i=0; i<stringLen; i++) {
                if(tempStr2[i] != ' ') {
                    tempStr2[j] = tempStr2[i];
                    j=j+1;
                }
            }
            tempStr2[j] = '\0';

            // Sort string into an array of strings
            status =  string_toStringDynamicArray(tempStr2, numVariable, variableName);

            if (tempStr2 != NULL) EG_free(tempStr2);
            tempStr2 = NULL;

            if (status != CAPS_SUCCESS) return status;

            // Print out list of variables found in load file
            printf("Variables found in file %s:\n",filename);
            for (i = 0; i < *numVariable; i++) printf("Variable %d = %s\n", i, (*variableName)[i]);
        // }

        // Get the number of data points in file if available in file line
        if (strncmp("zone t=", line, strlen("zone t=")) == 0) {

            // Pull out substring at first occurrence of i=
            tempStr = strstr(line, "i=");

            // Retrieve the i= value
            sscanf(&tempStr[2], "%d", numDataPoint);

            // Print out the number of data points found in load file
            printf("Number of data points = %d, in file %s\n", *numDataPoint, filename);
        } else {
            *numDataPoint = number_of_lines;
        }

    }

    if (*numVariable != 0 && *numDataPoint != 0) {

        // Allocate dataMatrix array
        if (*dataMatrix != NULL) EG_free(*dataMatrix);

        *dataMatrix = (double **) EG_alloc(*numVariable *sizeof(double *));
        if (*dataMatrix == NULL) return EGADS_MALLOC; // If allocation failed ....

        for (i = 0; i < *numVariable; i++) {

            (*dataMatrix)[i] = (double *) EG_alloc(*numDataPoint*sizeof(double));

            if ((*dataMatrix)[i] == NULL) { // If allocation failed ....
                for (j = 0; j < *numVariable; j++) {

                    if ((*dataMatrix)[j] != NULL ) EG_free((*dataMatrix)[j]);
                }

                if ((*dataMatrix) != NULL) EG_free((*dataMatrix));
                fclose(fp);
                return EGADS_MALLOC;
            }
        }

        // Loop through the file and fill up the data matrix
        for (j = 0; j < *numDataPoint; j++) {
            for (i = 0; i < *numVariable; i++) {
                fscanf(fp, "%lf", &(*dataMatrix)[i][j]);
                fscanf(fp, "%s", comma);
            }
        }

        // Output the first row of the dataMatrix
        for (i = 0; i < *numVariable; i++) printf("Variable %d - %.6f\n", i, (*dataMatrix)[i][0]);

    } else {

        printf("No data values extracted from file - %s",filename);
        fclose(fp);
        return CAPS_BADVALUE;
    }

    fclose(fp);
    return CAPS_SUCCESS;

}
