// Mesh related utility functions - Written by Dr. Ryan Durscher AFRL/RQVC

#include "meshTypes.h"  // Bring in mesh structures
#include "capsTypes.h"  // Bring in CAPS types
#include "cfdTypes.h"   // Bring in cfd structures
#include "feaTypes.h"   // Bring in fea structures
#include "miscTypes.h"  // Bring in miscellanous structures

#ifdef __cplusplus
extern "C" {
#endif

int mesh_bodyTessellation(ego tess,
						  mapAttrToIndexStruct *attrMap,
						  int *numNodes,
						  double *xyzCoord[],
         	 	 	 	  int *numTriFace, int *triFaceConn[], int *triFaceCompID[],
						  int *numBndEdge, int *bndEdgeConn[], int *bndEdgeCompID[],
						  int *tessFaceQuadMap,
						  int *numQuadFace, int *quadFaceConn[], int *quadFaceCompID[]);

int mesh_genUnstrSurfaceMeshEGADS(ego body,
								  double tessParams[3],
								  mapAttrToIndexStruct *attrMap,
								  genUnstrSurfMeshStruct *surfMesh);

int write_AFLR3(char *fname,
                 int asciiFlag,
                 genUnstrMeshStruct *genUnstrMesh);

int write_SU2(char *fname,
               int asciiFlag,
               genUnstrMeshStruct *mesh,
               int numBnds,
               int *bndIds);

int write_VTK(char *fname,
              int asciiFlag,
              genUnstrMeshStruct *mesh);

int write_Tecplot_Tet(char *fname,
                      int asciiFlag,
                      genUnstrMeshStruct *mesh);

int write_MAPBC(char *fname,
                 int numBnds,
                 int *bndIds,
                 int *bndVals);

int write_STL(char   *fname,
              int    numTris,
              double *verts,
              int    *conn);

void get_Surface_Norm(double p1[3],
                      double p2[3],
                      double p3[3],
                      double norm[3]);

int write_NASTRAN(char *fname,
                  int asciiFlag,
                  genFEAMeshStruct *nasMesh,
				  feaFileTypeEnum gridFileType);

// Populate bndCondStruct boundary condition information - Boundary condition values get filled with 99
int populate_bndCondStruct_from_bcPropsStruct(cfdBCsStruct *bcProps,
                                              bndCondStruct *bndConds);

// Populate bndCondStruct boundary condition information from attribute map - Boundary condition values get filled with 99
int populate_bndCondStruct_from_mapAttrToIndexStruct(mapAttrToIndexStruct *attrMap,
                                                     bndCondStruct *bndConds);

// Initiate (0 out all values and NULL all pointers) a volume Mesh in the genUnstrMeshStruct structure format
int initiate_genUnstrMeshStruct(genUnstrMeshStruct *mesh);

// Destroy (0 out all values, free all arrays and NULL all pointers) a volume  in the genUnstrMeshStruct structure format
int destroy_genUnstrMeshStruct(genUnstrMeshStruct *mesh);

// Initiate (0 out all values and NULL all pointers) a surface mesh in the genUnstrSurfMeshStruct structure format
int initiate_genUnstrSurfMeshStruct(genUnstrSurfMeshStruct *surfaceMesh);

// Destroy (0 out all values, free all arrays and NULL all pointers) a surface mesh in the genUnstrSurfMeshStruct structure format
int destroy_genUnstrSurfMeshStruct(genUnstrSurfMeshStruct *surfaceMesh);

// Initiate (0 out all values and NULL all pointers) a bndCond in the bndCondStruct structure format
int initiate_bndCondStruct(bndCondStruct *bndCond);

// Destroy (0 out all values, free all arrays, and NULL all pointers) a bndCond in the bndCondStruct structure format
int destroy_bndCondStruct(bndCondStruct *bndCond);

// Initiate (0 out all values and NULL all pointers) a meshInput in the meshInputStruct structure format
int initiate_meshInputStruct(meshInputStruct *meshInput);

// Destroy (0 out all values and NULL all pointers) a meshInput in the meshInputStruct structure format
int destroy_meshInputStruct(meshInputStruct *meshInput);

// Lump multiple surface meshes in genUnstrSurfMeshStruct format into a single surface mesh in genUnstrSurfMeshStruct format
int lump_genUnstrSurfaceMesh(int numSurfMesh,
		                     genUnstrSurfMeshStruct surfMesh[],
							 genUnstrSurfMeshStruct *surfMeshComb);

// Plot array of surface meshes
int plot_SurfaceMesh(meshInputStruct meshInput,
                     int numSurfMesh,
                     genUnstrSurfMeshStruct *surfaceMesh);

// Extrude a surface mesh a single unit the length of extrusionLength - return a
// volume mesh, cell volume and left-handness is not checked
int extrude_SurfaceMesh(double extrusionLength,
						genUnstrSurfMeshStruct surfaceMesh,
						genUnstrMeshStruct *volumeMesh);

// Initiate (0 out all values and NULL all pointers) a FEA mesh in the genFEAMeshStruct structure format
int initiate_genFEAMeshStruct(genFEAMeshStruct *feaMesh);

// Destroy (0 out all values, free all arrays and NULL all pointers) a FEA mesh in the genFEAMeshStruct structure format
int destroy_genFEAMeshStruct(genFEAMeshStruct *feaMesh);

// Lump multiple FEA meshes in genFEAMeshStructStruct format into a single mesh
// IMPORTANT: PID numbers are already considered to be with respect to the cumulative bodies
int lump_genFEAMesh(int numMesh, genFEAMeshStruct feaMesh[], genFEAMeshStruct *feaMeshComb);


// Initiate (0 out all values and NULL all pointers) a bodyTessMapping in the bodyTessMappingStruct structure format
int initiate_bodyTessMappingStruct (bodyTessMappingStruct *bodyTessMapping);

// Destroy (0 out all values and NULL all pointers) a bodyTessMapping in the bodyTessMappingStruct structure format
int destroy_bodyTessMappingStruct (bodyTessMappingStruct *bodyTessMapping);

// Initiate (0 out all values and NULL all pointers) a meshProp in the meshSizingStruct structure format
int initiate_meshSizingStruct (meshSizingStruct *meshProp);

// Destroy (0 out all values and NULL all pointers) a meshProp in the meshSizingStruct structure format
int destroy_meshSizingStruct (meshSizingStruct *meshProp);

// Fill meshProps in a meshBCStruct format with mesh boundary condition information from incoming Mesh Sizing Tuple
int mesh_getSizingProp(int numTuple,
                   	   capsTuple meshBCTuple[],
					   mapAttrToIndexStruct *attrMap,
					   int *numMeshProp,
					   meshSizingStruct *meshProps[]);

// Modify the EGADS body tessellation based on given inputs
int mesh_modifyBodyTess(int numMeshProp,
						meshSizingStruct meshProp[],
						int minEdgePointGlobal,
						int maxEdgePointGlobal,
						double tessParamGlobal[],
						mapAttrToIndexStruct attrMap,
						int numBody,
						ego *bodies[]);

#ifdef __cplusplus
}
#endif
