/*
 *      CAPS: Computational Aircraft Prototype Syntheses
 *
 *             Basic build test
 *
 *      Copyright 2014-2016, Massachusetts Institute of Technology
 *      Licensed under The GNU Lesser General Public License, version 2.1
 *      See http://www.opensource.org/licenses/lgpl-2.1.php
 *
 */

#include "caps.h"


int main(int argc, char *argv[])
{
  int      stat, nAttr, ints[2] = {11, 12};
  double   val = 13.0;
  capsObj  pobj, attr;

  if ((argc < 2) || (argc > 3)) {
    printf(" usage: basic filename [capsname]!\n");
    return 1;
  }

  stat = caps_open(argv[1], "basic", &pobj);
  if (stat != CAPS_SUCCESS) {
    printf(" caps_open = %d\n", stat);
    return 1;
  }
  
  /* make Problem-level attributes if not already there */
  stat = caps_size(pobj, ATTRIBUTES, NONE, &nAttr);
  if (stat != CAPS_SUCCESS) {
    printf(" CAPS Error: Problem Object ret=%d from caps_size(Attribute)!\n",
           stat);
  } else {
    if (nAttr == 0) {
      stat = caps_makeValue(pobj, "IntAttr", USER, Integer, 2, 1, (void *) ints,
                            NULL, &attr);
      if (stat != CAPS_SUCCESS) {
        printf(" CAPS Error: Problem Int Attr ret=%d from caps_makeValue!\n",
               stat);
      } else {
        stat = caps_setAttr(pobj, NULL, attr);
        if (stat != CAPS_SUCCESS)
          printf(" CAPS Error: Problem Int Attr ret=%d from caps_setAttr!\n",
                 stat);
        stat = caps_delete(attr);
        if (stat != CAPS_SUCCESS)
          printf(" CAPS Error: Problem Int Attr ret=%d from caps_delete!\n",
                 stat);
      }
      stat = caps_makeValue(pobj, "RealAttrs", USER, Double, 1, 1,
                            (void *) &val, NULL, &attr);
      if (stat != CAPS_SUCCESS) {
        printf(" CAPS Error: Problem Real Attr ret=%d from caps_makeValue!\n",
               stat);
      } else {
        stat = caps_setAttr(pobj, "RealAttr", attr);
        if (stat != CAPS_SUCCESS)
          printf(" CAPS Error: Problem Real Attr ret=%d from caps_setAttr!\n",
                 stat);
        stat = caps_delete(attr);
        if (stat != CAPS_SUCCESS)
          printf(" CAPS Error: Problem Real Attr ret=%d from caps_delete!\n",
                 stat);
      }
      stat = caps_makeValue(pobj, "StrAttr", USER, String, 0, 0,
                            "StringAttribute", NULL, &attr);
      if (stat != CAPS_SUCCESS) {
        printf(" CAPS Error: Problem Str Attr ret=%d from caps_makeValue!\n",
               stat);
      } else {
        stat = caps_setAttr(pobj, NULL, attr);
        if (stat != CAPS_SUCCESS)
          printf(" CAPS Error: Problem Str Attr ret=%d from caps_setAttr!\n",
                 stat);
        stat = caps_delete(attr);
        if (stat != CAPS_SUCCESS)
          printf(" CAPS Error: Problem Str Attr ret=%d from caps_delete!\n",
                 stat);
      }

    }
  }
  
  /* output what we have */
  printf("\n");
  printObjects(pobj, 0);
  printf("\n");

  /* checkpoint the Problem */
  if (argc == 3) {
    stat = caps_save(pobj, argv[2]);
    if (stat != CAPS_SUCCESS) printf(" caps_save = %d\n", stat);
  }
  
  stat = caps_close(pobj);
  if (stat != CAPS_SUCCESS) printf(" caps_close = %d\n", stat);

  return 0;
}
