# Import pyCAPS class file
from pyCAPS import capsProblem

# Import os module   
try: 
    import os
except:
    print "Unable to import os module"
    raise SystemExit

try: 
    import shutil
except:
    print "Unable to import shutil module"
    raise SystemExit

# Initialize pyCAPS object - directory where the pyCAPS.so resides
myProblem = capsProblem(libDir = os.environ['ESP_ROOT'] + "/lib")

# Create working directory variable 
workDir = "AeroelasticModal"

# Create projectName vairbale
projectName = "aeroelasticModal"

# Set the number of transfer iterations
numTransferIteration = 5

# Load CSM file 
myProblem.loadCAPS("./csmData/aeroelasticDataTransferSimple.csm")

# Load AIMs 
myProblem.loadAIM(aim = "tetgenAIM", 
                  altName= "tetgen",
                  analysisDir = workDir + "_FUN3D",
                  capsFidelity = "CFD")

  
myProblem.loadAIM(aim = "fun3dAIM", 
                  altName = "fun3d", 
                  analysisDir = workDir + "_FUN3D", 
                  parents = ["tetgen"],
                  capsFidelity = "CFD")

myProblem.loadAIM(aim = "mystranAIM", 
                  altName = "mystran", 
                  analysisDir = workDir + "_MYSTRAN",
                  capsFidelity = "STRUCTURE")

# Create an array of EigenVector names 
numEigenVector = 4
eigenVector = []
eigenRank = []
for i in range(numEigenVector):
    eigenVector.append("EigenVector_" + str(i+1))
    eigenRank.append(3)
    
transfers = ["Skin_Top", "Skin_Bottom", "Skin_Tip"]
for i in transfers:
    myProblem.createDataTransfer(variableName = eigenVector,
                                 variableRank = eigenRank, 
                                 #aimSrc = ["mystran"], # Take advantage of auto-assign - last AIM loaded
                                 #aimDest =["fun3d"], # Take advantage of auto-assign - second to last AIM loaded 
                                 #transferMethod = , #["Conserve", "Interpolate"] # Auto-assign Interpolate
                                 capsTransfer = i)

# Set inputs for tetgen 
myProblem.analysis["tetgen"].setAnalysisVal("Tess_Params", [.05, 0.01, 20.0])
myProblem.analysis["tetgen"].setAnalysisVal("Preserve_Surf_Mesh", True)

# Set inputs for fun3d
speedofSound = 340.0 # m/s
refVelocity = 100.0 # m/s
refDensity = 1.2 # kg/m^3

myProblem.analysis["fun3d"].setAnalysisVal("Proj_Name", projectName)
myProblem.analysis["fun3d"].setAnalysisVal("Mesh_ASCII_Flag", False)
myProblem.analysis["fun3d"].setAnalysisVal("Mach", refVelocity/speedofSound)
myProblem.analysis["fun3d"].setAnalysisVal("Eqn","compressible")
myProblem.analysis["fun3d"].setAnalysisVal("Viscous", "inviscid")
myProblem.analysis["fun3d"].setAnalysisVal("Num_Iter",10)
myProblem.analysis["fun3d"].setAnalysisVal("Time_Accuracy","2ndorderOPT")
myProblem.analysis["fun3d"].setAnalysisVal("Time_Step",0.001*speedofSound)
myProblem.analysis["fun3d"].setAnalysisVal("Num_Subiter", 30)
myProblem.analysis["fun3d"].setAnalysisVal("Temporal_Error",0.01)

myProblem.analysis["fun3d"].setAnalysisVal("CFL_Schedule",[1, 5.0])
myProblem.analysis["fun3d"].setAnalysisVal("CFL_Schedule_Iter", [1, 40])
myProblem.analysis["fun3d"].setAnalysisVal("Overwrite_NML", True)
myProblem.analysis["fun3d"].setAnalysisVal("Restart_Read","off")

inviscid = {"bcType" : "Inviscid", "wallTemperature" : 1.1}
myProblem.analysis["fun3d"].setAnalysisVal("Boundary_Condition", [("Skin", inviscid),
                                                                  ("SymmPlane", "SymmetryY"),
                                                                  ("Farfield","farfield")])

# Set inputs for mystran
myProblem.analysis["mystran"].setAnalysisVal("Proj_Name", projectName)
myProblem.analysis["mystran"].setAnalysisVal("Edge_Point_Max", 10)

myProblem.analysis["mystran"].setAnalysisVal("No_Quad_Faces", True) # Data transfer only supports triangles currently
myProblem.analysis["mystran"].setAnalysisVal("Tess_Params", [.5, .1, 15])

myProblem.analysis["mystran"].setAnalysisVal("Analysis_Type", "Modal");

eigen = { "extractionMethod"     : "MGIV", # "Lanczos",   
          "frequencyRange"       : [0.1, 200], 
          "numEstEigenvalue"     : 1,
          "numDesiredEigenvalue" : 5, 
          "eigenNormaliztion"    : "MASS", 
          "lanczosMode"          : 2,  # Default - not necesssary
          "lanczosType"          : "DPB"} # Default - not necesssary

myProblem.analysis["mystran"].setAnalysisVal("Analysis", ("EigenAnalysis", eigen))


madeupium    = {"materialType" : "isotropic", 
                "youngModulus" : 72.0E9 , 
                "poissonRatio": 0.33, 
                "density" : 2.8E3}
myProblem.analysis["mystran"].setAnalysisVal("Material", ("Madeupium", madeupium))

skin  = {"propertyType" : "Shell", 
         "membraneThickness" : 0.0015, 
         "material"        : "madeupium", 
         "bendingInertiaRatio" : 1.0, # Default           
         "shearMembraneRatio"  : 5.0/6.0} # Default 

ribSpar  = {"propertyType" : "Shell", 
            "membraneThickness" : 0.05, 
            "material"        : "madeupium", 
            "bendingInertiaRatio" : 1.0, # Default           
            "shearMembraneRatio"  : 5.0/6.0} # Default 

myProblem.analysis["mystran"].setAnalysisVal("Property", [("Skin", skin),
                                                          ("Rib_Root", ribSpar)])

constraint = {"groupName" : "Rib_Root", 
              "dofConstraint" : 123456}
myProblem.analysis["mystran"].setAnalysisVal("Constraint", ("edgeConstraint", constraint))



# Run pre/post-analysis for tetgen and fun3d - execute mystran
aim = ["tetgen", "mystran"]
for i in aim:
    print "\nRunning PreAnalysis ......" + i
    myProblem.analysis[i].aimPreAnalysis()

    if "mystran" in i:
     
        # Run mystran
        print "\n\nRunning Mystran......"  
        currentDirectory = os.getcwd() # Get our current working directory 

        os.chdir(myProblem.analysis[i].analysisDir) # Move into test directory

        os.system("mystran.exe " + projectName +  ".dat > Info.out") # Run fun3d via system call

        if os.path.getsize("Info.out") == 0: # 
            print "Mystran excution possibly failed\n"
            #myProblem.closeCAPS()
    
        os.chdir(currentDirectory) # Move back to top directory 
        
    if "tetgen" in i:    
        print "\nRunning PostAnalysis ......" + i
        # Run AIM post-analysis
        myProblem.analysis[i].aimPostAnalysis()

# Run aim pre-analysis/aim post-analysis - get ready for data transfer
aim = ["fun3d"]
for i in aim:
    print "\nRunning PreAnalysis ......" + i
    myProblem.analysis[i].aimPreAnalysis()
    
    print "\nRunning PostAnalysis ......" + i
    myProblem.analysis[i].aimPostAnalysis()

aim = ["mystran", "fun3d"]
for i in aim:
    
    if "mystran" in i:
        print "\nRunning PostAnalysis ......" + i
        # Run AIM post-analysis
        myProblem.analysis[i].aimPostAnalysis()

        #Execute the dataTransfer
        print "\nExecuting dataTransfer ......"
        for j in transfers:
            for eigenName in eigenVector:
                myProblem.dataTransfer[j].executeTransfer(eigenName)
        
        # Retrive natural frequencies from the structural analysis
        naturalFreq = myProblem.analysis[i].getAnalysisOutVal("EigenRadian") # rads/s
        mass = myProblem.analysis[i].getAnalysisOutVal("EigenGeneralMass")
    if "fun3d" in i:
        
        modalTuple = [] # Build up Modal Aeroelastic tuple  
        for j in eigenVector:
            modeNum = int(j.strip("EigenVector_"))
            #print "ModeNumber = ", modeNum 
            
            value = {"frequency" : naturalFreq[modeNum-1], 
                     "damping" : 0.99, 
                     "generalMass" : mass[modeNum-1], 
                     "generalVelocity" : 0.1}
            #print value
            modalTuple.append((j,value))
            
        # Add Eigen information in fun3d     
        myProblem.analysis[i].setAnalysisVal("Modal_Aeroelastic", modalTuple)
        myProblem.analysis[i].setAnalysisVal("Modal_Ref_Velocity", refVelocity)
        myProblem.analysis[i].setAnalysisVal("Modal_Ref_Dynamic_Pressure", 0.5*refDensity*refVelocity*refVelocity)
        myProblem.analysis[i].setAnalysisVal("Modal_Ref_Length", 1.0)
                
        # Re-run the preAnalysis 
        print "\nRunning PreAnalysis ......" + i
        myProblem.analysis[i].aimPreAnalysis()

        ####### Run fun3d ####################
        print "\n\nRunning FUN3D......"  
        currentDirectory = os.getcwd() # Get our current working directory 
        
        os.chdir(myProblem.analysis[i].analysisDir) # Move into test directory
        
        cmdLineOpt = "--moving_grid --aeroelastic_internal --animation_freq -1"
    
        os.system("mpirun -np 5 nodet_mpi " + cmdLineOpt + " > Info.out"); # Run fun3d via system call
            
        if os.path.getsize("Info.out") == 0: # 
            print "FUN3D excution failed\n"
            #myProblem.closeCAPS()
        
        os.chdir(currentDirectory) # Move back to top directory
        
        print "\nRunning PostAnalysis ......" + i
        # Run AIM post-analysis
        myProblem.analysis[i].aimPostAnalysis()

# Close CAPS 
myProblem.closeCAPS()