# Import pyCAPS class file
from pyCAPS import capsProblem

# Import os module   
try: 
    import os
except:
    print "Unable to import os module"
    raise SystemError

try: 
    import argparse
except: 
    print "Unable to import argparse module"
    raise SystemError

# Setup and read command line options. Please note that this isn't required for pyCAPS
parser = argparse.ArgumentParser(description = 'FUN3D Pytest Example',
                                 prog = 'fun3d_PyTest',
                                 formatter_class = argparse.ArgumentDefaultsHelpFormatter)

#Setup the available commandline options
parser.add_argument('-workDir', default = "./", nargs=1, type=str, help = 'Set working/run directory')
args = parser.parse_args()

# Initialize pyCAPS object - directory where the pyCAPS.so resides
myProblem = capsProblem(libDir = os.environ['ESP_ROOT'] + "/lib")

# Load CSM file 
myProblem.loadCAPS("./csmData/cfdMultiBody.csm")

# Load AFLR4 aim 
myProblem.loadAIM(aim = "aflr3AIM", 
                  altName = "aflr3", 
                  analysisDir=  str(args.workDir[0]) + "/AFLR3AnalysisTest", 
                  capsFidelity = "CFD")

# Set project name so a mesh file is generated 
myProblem.analysis["aflr3"].setAnalysisVal("Proj_Name", "pyCAPS_AFLR3_Test") 

#myProblem.analysis["aflr3"].setAnalysisVal("Tess_Params", [50, .11, 20.0])

myProblem.analysis["aflr3"].setAnalysisVal("Mesh_ASCII_Flag", True)

myProblem.analysis["aflr3"].setAnalysisVal("Mesh_Format", "AFLR3")

myProblem.analysis["aflr3"].setAnalysisVal("Mesh_Quiet_Flag", False)

useGlobal = False

if useGlobal:
    myProblem.analysis["aflr3"].setAnalysisVal("BL_Initial_Spacing", 0.1)
    myProblem.analysis["aflr3"].setAnalysisVal("BL_Thickness", 1.0)
    
else:
    inviscidBC = {"boundaryLayerThickness" : 0.0, "boundaryLayerSpacing" : 0.0}
    viscousBC  = {"boundaryLayerThickness" : 5.0, "boundaryLayerSpacing" : 0.05}
    
    # Set mesh sizing parmeters
    myProblem.analysis["aflr3"].setAnalysisVal("Mesh_Sizing", [("Wing1", inviscidBC),
                                                               ("Wing2", viscousBC)])

# Get analysis info. 
myProblem.analysis["aflr3"].getAnalysisInfo()

# Run AIM pre-analysis
myProblem.analysis["aflr3"].aimPreAnalysis()

#################################################
##  AFLR3  is internally ran during preAnalysis #
#################################################

# Run AIM post-analysis
myProblem.analysis["aflr3"].aimPostAnalysis()

# Close CAPS 
myProblem.closeCAPS()