# Import pyCAPS class file
from pyCAPS import capsProblem

# Import os module   
try: 
    import os
except:
    print "Unable to import os module"
    raise SystemError

try: 
    import argparse
except: 
    print "Unable to import argparse module"
    raise SystemError

# Setup and read command line options. Please note that this isn't required for pyCAPS
parser = argparse.ArgumentParser(description = 'AFLR4 Pytest Example',
                                 prog = 'aflr4_PyTest',
                                 formatter_class = argparse.ArgumentDefaultsHelpFormatter)

#Setup the available commandline options
parser.add_argument('-workDir', default = "./", nargs=1, type=str, help = 'Set working/run directory')
args = parser.parse_args()

# Initialize pyCAPS object - directory where the pyCAPS.so resides
myProblem = capsProblem(libDir = os.environ['ESP_ROOT'] + "/lib")

# Load CSM file 
myProblem.loadCAPS("./csmData/cfdMultiBody.csm")

# Load AFLR4 aim 
myProblem.loadAIM(aim = "aflr4AIM", altName = "aflr4", analysisDir= str(args.workDir[0]) + "/AFLR4AnalysisTest", capsFidelity = "CFD")

# Set project name so a mesh file is generated 
myProblem.analysis["aflr4"].setAnalysisVal("Proj_Name", "pyCAPS_AFLR4_Test") 

# Set output grid format since a project name is being supplied - Tecplot  file
myProblem.analysis["aflr4"].setAnalysisVal("Mesh_Format", "Tecplot")

# Run AIM pre-analysis
myProblem.analysis["aflr4"].aimPreAnalysis()

#######################################
## AFRL4 was ran during preAnalysis ##
#######################################

# Run AIM post-analysis
myProblem.analysis["aflr4"].aimPostAnalysis()

# Close CAPS 
myProblem.closeCAPS()

