# Import pyCAPS class file
from pyCAPS import capsProblem

# Import os module   
try: 
    import os
except:
    print "Unable to import os module"
    raise SystemError

try: 
    import argparse
except: 
    print "Unable to import argparse module"
    raise SystemError

# Setup and read command line options. Please note that this isn't required for pyCAPS
parser = argparse.ArgumentParser(description = 'AFLR4 and Tetgen Pytest Example',
                                 prog = 'aflr4_and_Tetgen_PyTest',
                                 formatter_class = argparse.ArgumentDefaultsHelpFormatter)

#Setup the available commandline options
parser.add_argument('-workDir', default = "./", nargs=1, type=str, help = 'Set working/run directory')
args = parser.parse_args()

# Set analysis directory 
directory = str(args.workDir[0]) + "/AFLR4TetgenAnalysisTest"

# Initialize pyCAPS object - directory where the pyCAPS.so resides
myProblem = capsProblem(libDir = os.environ['ESP_ROOT'] + "/lib")

# Load CSM file 
myProblem.loadCAPS("./csmData/cfdMultiBody.csm")

# Load AFLR4 aim 
myProblem.loadAIM(aim = "aflr4AIM", altName = "aflr4", analysisDir= directory , capsFidelity = "CFD")

# Run AIM pre-analysis
myProblem.analysis["aflr4"].aimPreAnalysis()

#######################################
## AFRL4 was ran during preAnalysis ##
#######################################

# Run AIM post-analysis
myProblem.analysis["aflr4"].aimPostAnalysis()

#######################################
## Build volume mesh off of surface  ##
##  mesh(es) using TetGen            ##
#######################################
 
# Load TetGen aim 
myProblem.loadAIM(aim = "tetgenAIM", altName = "tetgen", analysisDir= directory, parents = ["aflr4"])

# Set surface mesh preservation
myProblem.analysis["tetgen"].setAnalysisVal("Preserve_Surf_Mesh", True)

# Set project name - so a mesh file is generated 
myProblem.analysis["tetgen"].setAnalysisVal("Proj_Name", "pyCAPS_AFLR4_Tetgen_Test")

# Set output grid format since a project name is being supplied - Tecplot tetrahedral file
myProblem.analysis["tetgen"].setAnalysisVal("Mesh_Format", "Tecplot")

# Don't extract the update the local-to-global node connectivity
myProblem.analysis["tetgen"].setAnalysisVal("Ignore_Surface_Mesh_Extraction", True)

# Run AIM pre-analysis
myProblem.analysis["tetgen"].aimPreAnalysis()

#######################################
## Tetgen was ran during preAnalysis ##
#######################################

# Run AIM post-analysis
myProblem.analysis["tetgen"].aimPostAnalysis()

# Close CAPS 
myProblem.closeCAPS()
