# Import myProblem class file
from pyCAPS import capsProblem

# Import os module
try:
    import os
except:
    print "Unable to import os module"
    raise SystemExit

# Initialize myProblem object - directory where the pyCAPS.so resides
myProblem = capsProblem(libDir = os.environ['ESP_ROOT'] + "/lib")

# Load CSM file
myProblem.loadCAPS("./csmData/avlWingTail.csm")

# Set fidelity - could also be set during .loadAIM
myProblem.capsFidelity = "LINEARAERO"

# Change a design parameter - area in the geometry 
myProblem.geometry.setGeometryVal("area", 10.0)

# Load desired aim 
print "Loading AIM"
myProblem.loadAIM(aim = "avlAIM", analysisDir = "AVLPyTest")

print "Setting design parameters"

# Set new Mach/Alt parameters
myProblem.analysis["avlAIM"].setAnalysisVal("Mach", 0.5)
myProblem.analysis["avlAIM"].setAnalysisVal("Alpha", 1.0)
myProblem.analysis["avlAIM"].setAnalysisVal("Beta", 0.0)
myProblem.analysis["avlAIM"].setAnalysisVal("RollRate", 0.0)
myProblem.analysis["avlAIM"].setAnalysisVal("PitchRate", 0.0)
myProblem.analysis["avlAIM"].setAnalysisVal("YawRate", 0.0)

wing = {"groupName"  : "Wing", 
        "numChord"   : 8, 
        "spaceChord" : 1.0, 
        "numSpan"    : 12, 
        "spaceSpan"  : 1.0}

tail = {"numChord"   : 8, 
        "spaceChord" : 1.0, 
        "numSpan"    : 12, 
        "spaceSpan"  : 1.0}

myProblem.analysis["avlAIM"].setAnalysisVal("AVL_Surface", [("Wing", wing), 
                                                            ("Vertical_Tail", tail)])


# Set control surface parameters
flapLE = {"controlGain"    : 0.5, 
          "deflectionAngle" : 25.0}

flapTE = {"controlGain"    : 1.0,
          "deflectionAngle" : 15.0}


myProblem.analysis["avlAIM"].setAnalysisVal("AVL_Control", [("WingRightLE", flapLE), 
                                                            ("WingRightTE", flapTE),
                                                            #("WingLeftLE", flapLE), # Don't change anything
                                                            #("WingLeftTE", flapTE)
                                                            ("Tail"  , flapTE)])

# Run AIM pre-analysis
myProblem.analysis["avlAIM"].aimPreAnalysis()

# Run AVL
print "Running AVL"
currentDirectory = os.getcwd() # Get our current working directory 

os.chdir(myProblem.analysis["avlAIM"].analysisDir) # Move into test directory
os.system("avl caps < avlInput.txt > avlOutput.txt");

os.chdir(currentDirectory) # Move back to working directory 

# Run AIM post-analysis
myProblem.analysis["avlAIM"].aimPostAnalysis()

# Get Output Data from AVL
print "Alpha  " + str(myProblem.analysis["avlAIM"].getAnalysisOutVal("Alpha"));
print "Beta   " + str(myProblem.analysis["avlAIM"].getAnalysisOutVal("Beta"));
print "Mach   " + str(myProblem.analysis["avlAIM"].getAnalysisOutVal("Mach"));
print "pb/2V  " + str(myProblem.analysis["avlAIM"].getAnalysisOutVal("pb/2V"));
print "qc/2V  " + str(myProblem.analysis["avlAIM"].getAnalysisOutVal("qc/2V"));
print "rb/2V  " + str(myProblem.analysis["avlAIM"].getAnalysisOutVal("rb/2V"));
print "p'b/2V " + str(myProblem.analysis["avlAIM"].getAnalysisOutVal("p'b/2V"));
print "r'b/2V " + str(myProblem.analysis["avlAIM"].getAnalysisOutVal("r'b/2V"));
print "CXtot  " + str(myProblem.analysis["avlAIM"].getAnalysisOutVal("CXtot"));
print "CYtot  " + str(myProblem.analysis["avlAIM"].getAnalysisOutVal("CYtot"));
print "CZtot  " + str(myProblem.analysis["avlAIM"].getAnalysisOutVal("CZtot"));
print "Cltot  " + str(myProblem.analysis["avlAIM"].getAnalysisOutVal("Cltot"));
print "Cmtot  " + str(myProblem.analysis["avlAIM"].getAnalysisOutVal("Cmtot"));
print "Cntot  " + str(myProblem.analysis["avlAIM"].getAnalysisOutVal("Cntot"));
print "Cl'tot " + str(myProblem.analysis["avlAIM"].getAnalysisOutVal("Cl'tot"));
print "Cn'tot " + str(myProblem.analysis["avlAIM"].getAnalysisOutVal("Cn'tot"));
print "CLtot  " + str(myProblem.analysis["avlAIM"].getAnalysisOutVal("CLtot"));
print "CDtot  " + str(myProblem.analysis["avlAIM"].getAnalysisOutVal("CDtot"));
print "CDvis  " + str(myProblem.analysis["avlAIM"].getAnalysisOutVal("CDvis"));
print "CLff   " + str(myProblem.analysis["avlAIM"].getAnalysisOutVal("CLff"));
print "CYff   " + str(myProblem.analysis["avlAIM"].getAnalysisOutVal("CYff"));
print "CDind  " + str(myProblem.analysis["avlAIM"].getAnalysisOutVal("CDind"));
print "CDff   " + str(myProblem.analysis["avlAIM"].getAnalysisOutVal("CDff"));
print "e      " + str(myProblem.analysis["avlAIM"].getAnalysisOutVal("e"));

# Save away CAPS problem
#myProblem.saveCAPS()

# Close CAPS 
myProblem.closeCAPS()
