# Import myProblem class file
from pyCAPS import capsProblem

# Import os module
try:
    import os
except:
    print "Unable to import os module"
    raise SystemExit

# Initialize myProblem object - directory where the myProblem.so resides
myProblem = capsProblem(libDir = os.environ['ESP_ROOT'] + "/lib")

# Load CSM file
myProblem.loadCAPS("./csmData/awaveWingTailFuselage.csm")

# Set fidelity - could also be set during .loadAIM
myProblem.capsFidelity = "LINEARAERO"

# Change a design parameter - area in the geometry 
myProblem.geometry.setGeometryVal("area", 10.0)

# Load desired aim 
myProblem.loadAIM(aim = "awaveAIM", analysisDir = "AwavePyTest")

# Set new Mach/Alt parameters
myProblem.analysis["awaveAIM"].setAnalysisVal("Mach", [ 1.2, 1.5])
myProblem.analysis["awaveAIM"].setAnalysisVal("AoA", [ 0.0, 2.0])

# Run AIM pre-analysis
myProblem.analysis["awaveAIM"].aimPreAnalysis()

# Run AIM
print " Running AWAVE "
currentDirectory = os.getcwd() # Get our current working directory 

os.chdir(myProblem.analysis["awaveAIM"].analysisDir) # Move into test directory
os.system("wdexe awaveInput.txt");

os.chdir(currentDirectory) # Move back to working directory 

# Run AIM post-analysis
print " Running Post Analysis"
myProblem.analysis["awaveAIM"].aimPostAnalysis()

myProblem.analysis["awaveAIM"].getAnalysisOutVal("MachOut");
myProblem.analysis["awaveAIM"].getAnalysisOutVal("CDwave");
myProblem.analysis["awaveAIM"].getAnalysisOutVal("AoAOut");

# Save away CAPS problem
#myProblem.saveCAPS()

# Close CAPS
print " Calling closeCAPS"
myProblem.closeCAPS()
