# Import pyCAPS class file
from pyCAPS import capsProblem

# Import os module
try:
    import os
except:
    print "Unable to import os module"
    raise SystemExit

# Initialize myProblem object - directory where the pyCAPS.so resides
myProblem = capsProblem(libDir = os.environ['ESP_ROOT'] + "/lib")

# Load CSM file
myProblem.loadCAPS("./csmData/frictionWingTailFuselage.csm")

# Set fidelity - could also be set during .loadAIM
myProblem.capsFidelity = "LINEARAERO"

# Change a design parameter - area in the geometry 
myProblem.geometry.setGeometryVal("area", 10.0)

# Load desired aim 
print "Loading AIM"
myProblem.loadAIM(aim = "frictionAIM", analysisDir = "FrictionPyTest")

# Set new Mach/Alt parameters
print "Setting Mach number"
myProblem.analysis["frictionAIM"].setAnalysisVal("Mach", [0.5, 1.5])

print "Setting Altitude"
myProblem.analysis["frictionAIM"].setAnalysisVal("Altitude", [30., 60.])

# Run AIM pre-analysis
myProblem.analysis["frictionAIM"].aimPreAnalysis()

# Run Friction
currentDirectory = os.getcwd() # Get our current working directory 

os.chdir(myProblem.analysis["frictionAIM"].analysisDir) # Move into test directory
os.system("friction frictionInput.txt frictionOutput.txt");

os.chdir(currentDirectory) # Move back to working directory 

# Run AIM post-analysis
myProblem.analysis["frictionAIM"].aimPostAnalysis()

# Get Output Data from Friction
myProblem.analysis["frictionAIM"].getAnalysisOutVal("CDtotal");
myProblem.analysis["frictionAIM"].getAnalysisOutVal("CDform");
myProblem.analysis["frictionAIM"].getAnalysisOutVal("CDfric");

# Close CAPS 
myProblem.closeCAPS()
