# Import pyCAPS class file
from pyCAPS import capsProblem

# Import os module   
try: 
    import os
except:
    print "Unable to import os module"
    raise SystemExit

# Initialize pyCAPS object - directory where the pyCAPS.so resides
myProblem = capsProblem(libDir = os.environ['ESP_ROOT'] + "/lib")

# Create working directory variable 
workDir = "FUN3DTetgenAlphaSweep"

# Load CSM file 
myProblem.loadCAPS("./csmData/cfdMultiBody.csm")

# Set fidelity - could also be set during .loadAIM
myProblem.capsFidelity = "ALL"

# Load Tetgen aim 
myProblem.loadAIM(aim = "tetgenAIM", analysisDir = workDir)

# Set new EGADS body tessellation parameters
myProblem.analysis["tetgenAIM"].setAnalysisVal("Tess_Params", [.05, 0.01, 20.0])

# Preserve surface mesh while messhing
myProblem.analysis["tetgenAIM"].setAnalysisVal("Preserve_Surf_Mesh", True)

# Get analysis info. 
myProblem.analysis["tetgenAIM"].getAnalysisInfo()

# Run AIM pre-analysis
myProblem.analysis["tetgenAIM"].aimPreAnalysis()

# NO analysis is needed - Tetgen was already ran dureing preAnalysis

# Run AIM post-analysis
myProblem.analysis["tetgenAIM"].aimPostAnalysis()

# Get analysis info. 
myProblem.analysis["tetgenAIM"].getAnalysisInfo()

Alpha = [-5, 0, 5, 10]
Cl = []
Cd = []
for a in Alpha:
    analysisID = "Alpha_" + str(a)
    # Load FUN3D aim - child of Tetgen AIM  
    myProblem.loadAIM(aim = "fun3dAIM", 
                      altName = analysisID, 
                      analysisDir = workDir + "/" + analysisID, parents = ["tetgenAIM"])

    # Set project name
    myProblem.analysis[analysisID].setAnalysisVal("Proj_Name", analysisID)

    myProblem.analysis[analysisID].setAnalysisVal("Mesh_ASCII_Flag", False)
  
    # Set AoA number 
    myProblem.analysis[analysisID].setAnalysisVal("Alpha", a) 

    # Set Mach number 
    myProblem.analysis[analysisID].setAnalysisVal("Mach", 0.25)

    # Set equation type
    myProblem.analysis[analysisID].setAnalysisVal("Eqn","compressible")
    
    # Set Viscous term      
    myProblem.analysis[analysisID].setAnalysisVal("Viscous", "inviscid")

    # Set number of iterations
    myProblem.analysis[analysisID].setAnalysisVal("Num_Iter",10)

    # Set CFL number schedule
    myProblem.analysis[analysisID].setAnalysisVal("CFL_Schedule",[0.5, 3.0])

    # Set read restart option
    myProblem.analysis[analysisID].setAnalysisVal("Restart_Read","off")

    # Set CFL number iteration schedule
    myProblem.analysis[analysisID].setAnalysisVal("CFL_Schedule_Iter", [1, 40])
 
    # Set overwrite fun3d.nml if not linking to Python library
    myProblem.analysis[analysisID].setAnalysisVal("Overwrite_NML", True)

    # Set boundary conditions
    inviscidBC1 = {"bcType" : "Inviscid", "wallTemperature" : 1}
    inviscidBC2 = {"bcType" : "Inviscid", "wallTemperature" : 1.2}
    myProblem.analysis[analysisID].setAnalysisVal("Boundary_Condition", [("Wing1", inviscidBC1),
                                                                         ("Wing2", inviscidBC2),
                                                                         ("Farfield","farfield")])
 
    # Run AIM pre-analysis
    myProblem.analysis[analysisID].aimPreAnalysis()

    ####### Run fun3d ####################
    print "\n\nRunning FUN3D......"  
    currentDirectory = os.getcwd() # Get our current working directory 

    os.chdir(myProblem.analysis[analysisID].analysisDir) # Move into test directory

    os.system("nodet_mpi --animation_freq -1000 > Info.out"); # Run fun3d via system call

    haveFUN3D = True # See if FUN3D ran
    if os.path.getsize("Info.out") == 0: # 
        print "FUN3D excution failed\n"
        haveFUN3D = False
    
    os.chdir(currentDirectory) # Move back to top directory 
    #######################################

    # Run AIM post-analysis
    myProblem.analysis[analysisID].aimPostAnalysis()

    # Get force results
    if haveFUN3D:
        Cl.append(myProblem.analysis[analysisID].getAnalysisOutVal("CLtot"))
        Cd.append(myProblem.analysis[analysisID].getAnalysisOutVal("CDtot")) 

print "Alpha, CL, CD"
for i in range(len(Alpha)):
    print Alpha[i], Cl[i], Cd[i]
    
# Close CAPS 
myProblem.closeCAPS()
