# 

# Import pyCAPS class file
# Note: Either a copy or soft link of pyCAPS.py must be present in the same folder where this script is executed.
from pyCAPS import capsProblem

# Import os module   
try: 
    import os
except:
    print "Unable to import os module"
    raise SystemExit

#-------------------------------- Work Folder ----------------------------------#
# Note: Either a copy or soft link of pyCAPS.py must be present for execution


# Initialize pyCAPS object - directory where the pyCAPS.so resides
myProblem = capsProblem(libDir = os.environ['ESP_ROOT'] + "/lib")

# Project Name
projectName = "myGlider"


#--------------------------- Geometry Information ------------------------------#
print "Loading geometry"

# Load CSM file 
myProblem.loadCAPS(os.environ['ESP_ROOT']+"/../../OpenCSM/trunk/data/CAPS/myGlider.csm")

print "Loaded geometry"

# Set analysis fidelity 
myProblem.capsFidelity = "CFD"

print "Loaded CAPSfidelity"

# Get OML (geometry fildelity, csm file)
myProblem.geometry.setGeometryVal("fidelity", 3)

print "Loaded fidelity"

# Set Bounding box for farfield boundaries
myProblem.geometry.setGeometryVal("farfield:onoff", 1)

print "Loaded farfield"

# Set Control Surface deflection on Wing
myProblem.geometry.setGeometryVal("wing:angleft", 0.0)
myProblem.geometry.setGeometryVal("wing:angrite", 0.0)

print "Loaded wing angles"
# Set Control Surface deflection on Horizontal Tail
myProblem.geometry.setGeometryVal("htail:deflect", 0.0)

print "Loaded htail angles"
# Set Control Surface deflection on Vertical Tail
myProblem.geometry.setGeometryVal("vtail:deflect", 0.0)

print "Loaded vtail angles"


print "Saving geometry"
myProblem.geometry.saveGeometry(projectName, directory = projectName, extension="egads")
print "Saving done"

# exit(0)

#------------------------------- Mesh Information ------------------------------#

# Load Tetgen aim 
myProblem.loadAIM(aim = "tetgenAIM", analysisDir= projectName)

# # Set project name, this will dump volume mesh in the work folder
# myProblem.analysis["tetgenAIM"].setAnalysisVal("Proj_Name", projectName)

# Write volume mesh in VTK format (to check/debug)
myProblem.analysis["tetgenAIM"].setAnalysisVal("Mesh_Format", "VTK")

# Set new EGADS body tessellation parameters
myProblem.analysis["tetgenAIM"].setAnalysisVal("Tess_Params", [0.025, 0.1, 15.0])
# myProblem.analysis["tetgenAIM"].setAnalysisVal("Tess_Params", [0.0125, 0.1, 15.0])
# myProblem.analysis["tetgenAIM"].setAnalysisVal("Tess_Params", [0.00625, 0.1, 15.0])

# Set Volume mesh quality/control parameters at once
tetgen_params = "YCq1.2a166.67T1e-16"
# tetgen_params = "YCq1.2a41.667T1e-16"
# tetgen_params = "YCq1.2a10.41667T1e-16"
myProblem.analysis["tetgenAIM"].setAnalysisVal("Mesh_Gen_Input_String", tetgen_params)

# Get analysis info. 
myProblem.analysis["tetgenAIM"].getAnalysisInfo()

# Run AIM pre-analysis
myProblem.analysis["tetgenAIM"].aimPreAnalysis()


# Run AIM post-analysis
myProblem.analysis["tetgenAIM"].aimPostAnalysis()

# Get analysis info. 
myProblem.analysis["tetgenAIM"].getAnalysisInfo()

# DEBUG: Exit without running Flow analysis
# exit(0)


#--------------------------- Flow Solver Information ---------------------------#

# Load SU2 aim - child of Tetgen AIM  
myProblem.loadAIM(aim = "su2AIM", 
                  altName = "su2", 
                  analysisDir = projectName, parents = ["tetgenAIM"])
# Set project name
myProblem.analysis["su2"].setAnalysisVal("Proj_Name", projectName)

# myProblem.analysis["su2"].setAnalysisVal("Mesh_ASCII_Flag", False)
  
# Set AoA number 
myProblem.analysis["su2"].setAnalysisVal("Alpha", 0.0) 

# Set Mach number 
myProblem.analysis["su2"].setAnalysisVal("Mach", 0.3)

# Set equation type
myProblem.analysis["su2"].setAnalysisVal("Eqn","compressible")

# Set number of iterations
myProblem.analysis["su2"].setAnalysisVal("Num_Iter",10000)

# Set CFL number
myProblem.analysis["su2"].setAnalysisVal("CFL_NUMBER",5.0)

# Switch off Multigrid level
myProblem.analysis["su2"].setAnalysisVal("MGLEVEL",0)

# Set the Freestream conditions to an altitude of 5000ft.
myProblem.analysis["su2"].setAnalysisVal("SYSTEM_MEASUREMENTS","US")
myProblem.analysis["su2"].setAnalysisVal("FREESTREAM_PRESSURE",1760.8752)
myProblem.analysis["su2"].setAnalysisVal("FREESTREAM_TEMPERATURE", 500.843)
myProblem.analysis["su2"].setAnalysisVal("FREESTREAM_DENSITY",0.002048)
myProblem.analysis["su2"].setAnalysisVal("FREESTREAM_VISCOSITY",3.6365699E-7)

# Set read restart option
myProblem.analysis["su2"].setAnalysisVal("Restart_Read","off")

# Set overwrite SU2 configuration file
myProblem.analysis["su2"].setAnalysisVal("Overwrite_CFG", True)

# Set boundary conditions
inviscidBC = {"bcType" : "Inviscid"}
myProblem.analysis["su2"].setAnalysisVal("Boundary_Condition", [("glider", inviscidBC),
                                                                ("Farfield","Farfield")])
 
# Run AIM pre-analysis
myProblem.analysis["su2"].aimPreAnalysis()

####### Run su2 ####################
print "\n\nRunning SU2......"  

# Get our current working directory 
currentDirectory = os.getcwd() 

# Move into test directory
os.chdir(myProblem.analysis["su2"].analysisDir) 

# Set number of processors for SU2 parallel execution
Num_Procs = 16

# Run su2 via system call
os.system("parallel_computation.py -f "+projectName+".cfg -n "+str(Num_Procs)+" > Info.out"); 

# Check if system call for SU2 was successful
haveSU2 = True 
if os.path.getsize("Info.out") == 0: # 
    print "SU2 excution failed\n"
    haveSU2 = False
else: 
    print "......Done"

# Move back to top directory 
os.chdir(currentDirectory) 
#######################################

# Run AIM post-analysis
myProblem.analysis["su2"].aimPostAnalysis()

# Get force results
# if haveSU2: 
#     print "Total Force - Pressure + Viscous"
#     # Get Lift and Drag coefficients
#     print "Cl = " , myProblem.analysis["su2"].getAnalysisOutVal("CLtot"), \
#           "Cd = " , myProblem.analysis["su2"].getAnalysisOutVal("CDtot")
# 
#     # Get Cmx, Cmy, and Cmz coefficients 
#     print "Cmx = " , myProblem.analysis["su2"].getAnalysisOutVal("CMXtot"), \
#           "Cmy = " , myProblem.analysis["su2"].getAnalysisOutVal("CMYtot"), \
#           "Cmz = " , myProblem.analysis["su2"].getAnalysisOutVal("CMZtot")
#     
#     # Get Cx, Cy, Cz coefficients 
#     print "Cx = " , myProblem.analysis["su2"].getAnalysisOutVal("CXtot"), \
#           "Cy = " , myProblem.analysis["su2"].getAnalysisOutVal("CYtot"), \
#           "Cz = " , myProblem.analysis["su2"].getAnalysisOutVal("CZtot")
    
# Save away CAPS problem
#myProblem.saveCAPS()

# Close CAPS 
myProblem.closeCAPS()
#-------------------------------- End of Script  -------------------------------#
