# Import pyCAPS class file
from pyCAPS import capsProblem

# Import os module   
try: 
    import os
except:
    print "Unable to import os module"
    raise SystemExit

# Initialize pyCAPS object - directory where the pyCAPS.so resides
myProblem = capsProblem(libDir = os.environ['ESP_ROOT'] + "/lib")

# Load CSM file 
myProblem.loadCAPS("./csmData/feaAGARD445.csm")

# Create project name 
projectName = "MystranModalAGARD445"

# Change the sweepAngle and span of the Geometry - Demo purposes 
#myProblem.geometry.setGeometryVal("sweepAngle", 5) # From 45 to 5 degrees 
#myProblem.geometry.setGeometryVal("semiSpan", 5)   # From 2.5 ft to 5 ft

# Load mystran aim 
myProblem.loadAIM(aim = "mystranAIM", altName = "mystran", analysisDir = projectName, capsFidelity = "ALL")

# Set project name so a mesh file is generated 
myProblem.analysis["mystran"].setAnalysisVal("Proj_Name", projectName) 

# Set meshing parameters
myProblem.analysis["mystran"].setAnalysisVal("Edge_Point_Max", 10)
myProblem.analysis["mystran"].setAnalysisVal("Edge_Point_Min", 6)

myProblem.analysis["mystran"].setAnalysisVal("No_Quad_Faces", False)

myProblem.analysis["mystran"].setAnalysisVal("Tess_Params", [.25,.01,15])

# Set analysis type
myProblem.analysis["mystran"].setAnalysisVal("Analysis_Type", "Modal");

# Set analysis inputs
eigen = { "extractionMethod"     : "MGIV", # "Lanczos",   
          "frequencyRange"       : [0.1, 200], 
          "numEstEigenvalue"     : 1,
          "numDesiredEigenvalue" : 2, 
          "eigenNormaliztion"    : "MASS", 
	      "lanczosMode"          : 2,  # Default - not necesssary
          "lanczosType"          : "DPB"} # Default - not necesssary

myProblem.analysis["mystran"].setAnalysisVal("Analysis", ("EigenAnalysis", eigen))

# Set materials 
mahogany    = {"materialType"        : "orthotropic", 
               "youngModulus"        : 0.457E6 ,
               "youngModulusLateral" : 0.0636E6,
               "poissonRatio"        : 0.31,                
               "shearModulus"        : 0.0637E6,
               "shearModulusTrans1Z" : 0.00227E6,
               "shearModulusTrans2Z" : 0.00227E6,
               "density"             : 3.5742E-5}

myProblem.analysis["mystran"].setAnalysisVal("Material", ("Mahogany", mahogany))

# Set properties
shell  = {"propertyType" : "Shell", 
          "membraneThickness" : 0.82, 
          "material"        : "mahogany", 
          "bendingInertiaRatio" : 1.0, # Default - not necesssary           
          "shearMembraneRatio"  : 5.0/6.0} # Default - not necesssary 

myProblem.analysis["mystran"].setAnalysisVal("Property", ("yatesPlate", shell))

# Set constraints
constraint = {"groupName" : "constEdge",
              "dofConstraint" : 123456}

myProblem.analysis["mystran"].setAnalysisVal("Constraint", ("edgeConstraint", constraint))

# Run AIM pre-analysis
myProblem.analysis["mystran"].aimPreAnalysis()

####### Run MYSTRAN ####################
print "\n\nRunning MYSTRAN......"  
currentDirectory = os.getcwd() # Get our current working directory 

os.chdir(myProblem.analysis["mystran"].analysisDir) # Move into test directory

os.system("mystran.exe " + projectName +  ".dat"); # Run mystran via system call

os.chdir(currentDirectory) # Move back to working directory 

print "Done running MYSTRAN!"
########################################

# Run AIM post-analysis
myProblem.analysis["mystran"].aimPostAnalysis()

# Get Eigen-frequencies
print "\nGetting results natural frequencies....." 
natrualFreq = myProblem.analysis["mystran"].getAnalysisOutVal("EigenFrequency")

mode = 1
for i in natrualFreq:
    print "Natural freq (Mode {:d}) = ".format(mode) + '{:.2f} '.format(i) + "(Hz)"
    mode += 1                 
                           
# Close CAPS 
myProblem.closeCAPS()
