# Import pyCAPS class file
from pyCAPS import capsProblem

# Import os module   
try: 
    import os
except:
    print "Unable to import os module"
    raise SystemExit

# Initialize pyCAPS object - directory where the pyCAPS.so resides
myProblem = capsProblem(libDir = os.environ['ESP_ROOT'] + "/lib")

# Load CSM file 
myProblem.loadCAPS("./csmData/feaWingBEM.csm")

# Load mystran aim 
myProblem.loadAIM(aim = "mystranAIM", altName = "mystran", analysisDir= "MystranModalWingBEM", capsFidelity = "ALL")

# Set project name so a mesh file is generated 
projectName = "pyCAPS_mystran_Test"
myProblem.analysis["mystran"].setAnalysisVal("Proj_Name", projectName) 

myProblem.analysis["mystran"].setAnalysisVal("Edge_Point_Max", 5)

myProblem.analysis["mystran"].setAnalysisVal("No_Quad_Faces", False)

# Set analysis 
eigen = { "extractionMethod"     : "GIV", 
          "frequencyRange"       : [0, 10000], 
          "numEstEigenvalue"     : 1,
          "numDesiredEigenvalue" : 10, 
          "eigenNormaliztion"    : "MASS"}
myProblem.analysis["mystran"].setAnalysisVal("Analysis", ("EigenAnalysis", eigen))

# Set materials
unobtainium  = {"youngModulus" : 2.2E6 , 
                "poissonRatio" : .5, 
                "density"      : 7850}

madeupium    = {"materialType" : "isotropic", 
                "youngModulus" : 1.2E5 , 
                "poissonRatio" : .5, 
                "density"      : 7850}
myProblem.analysis["mystran"].setAnalysisVal("Material", [("Unobtainium", unobtainium),
                                                          ("Madeupium", madeupium)])

# Set property 
shell  = {"propertyType"      : "Shell", 
          "membraneThickness" : 0.2,
          "bendingInertiaRatio" : 1.0, # Default           
          "shearMembraneRatio"  : 5.0/6.0} # Default }
 
shell2 = {"propertyType"      : "Shell", 
          "material"          : "madeupium", 
          "membraneThickness" : 0.4,
          "bendingInertiaRatio" : 1.0, # Default           
          "shearMembraneRatio"  : 5.0/6.0} # Default }

myProblem.analysis["mystran"].setAnalysisVal("Property", [("Ribs_and_Spars", shell),
                                                          ("Rib_Constraint", shell2)])

# Set constraints 
constraint = {"groupName" : ["Rib_Constraint"], 
              "dofConstraint" : 123456}

myProblem.analysis["mystran"].setAnalysisVal("Constraint", ("ribConstraint", constraint))


# Run AIM pre-analysis
myProblem.analysis["mystran"].aimPreAnalysis()

####### Run Mystran####################
print "\n\nRunning MYSTRAN......"  
currentDirectory = os.getcwd() # Get our current working directory 

os.chdir(myProblem.analysis["mystran"].analysisDir) # Move into test directory

os.system("mystran.exe " + projectName +  ".dat"); # Run mystran via system call

os.chdir(currentDirectory) # Move back to working directory 
print "Done running MYSTRAN!"

# Run AIM post-analysis
myProblem.analysis["mystran"].aimPostAnalysis()

# Close CAPS 
myProblem.closeCAPS()
