# Import pyCAPS class file
from pyCAPS import capsProblem

# Import os module   
try: 
    import os
except:
    print "Unable to import os module"
    raise SystemExit

# Initialize pyCAPS object - directory where the pyCAPS.so resides
myProblem = capsProblem(libDir = os.environ['ESP_ROOT'] + "/lib")

# Load CSM file 
myProblem.loadCAPS("./csmData/feaCantileverPlate.csm")

# Load nastran aim 
myProblem.loadAIM(aim = "nastranAIM", altName = "nastran", analysisDir= "NastranCompositeTest", capsFidelity = "STRUCTURE")

# Set project name so a mesh file is generated 
projectName = "pyCAPS_nastran_Test"
myProblem.analysis["nastran"].setAnalysisVal("Proj_Name", projectName) 

myProblem.analysis["nastran"].setAnalysisVal("Edge_Point_Max", 10)

myProblem.analysis["nastran"].setAnalysisVal("No_Quad_Faces", False)

myProblem.analysis["nastran"].setAnalysisVal("File_Format", "Small")

myProblem.analysis["nastran"].setAnalysisVal("Mesh_File_Format", "Large")

# Set analysis 
eigen = { "extractionMethod"     : "Lanczos", 
          "frequencyRange"       : [0, 10000], 
          "numEstEigenvalue"     : 1,
          "numDesiredEigenvalue" : 4, 
          "eigenNormaliztion"    : "MASS"}
myProblem.analysis["nastran"].setAnalysisVal("Analysis", ("EigenAnalysis", eigen))

# Set materials
unobtainium  = {"youngModulus" : 2.2E6 , 
                "poissonRatio" : .5, 
                "density"      : 7850}

madeupium    = {"materialType" : "isotropic", 
                "youngModulus" : 1.2E5 , 
                "poissonRatio" : .5, 
                "density"      : 7850}
myProblem.analysis["nastran"].setAnalysisVal("Material", [("Unobtainium", unobtainium),
                                                          ("Madeupium", madeupium)])

# Set property 
shell  = {"propertyType"      : "Composite",
          "shearBondAllowable": 1.0e6,
	      "bendingInertiaRatio" : 1.0, # Default - not necesssary           
          "shearMembraneRatio"  : 0, # Turn off shear - no materialShear
          "membraneThickness" : 0.2, 
	      "compositeMaterial" : ["Unobtainium", "Madeupium", "Madeupium"],
	      "compositeFailureTheory" :"STRN",
          "compositeThickness" : [1.2, 0.5, 2.0],
	      "compositeOrientation" : [30.6, 45, 50.4]}
 
myProblem.analysis["nastran"].setAnalysisVal("Property", ("plate", shell))
                                                         
# Set constraints 
constraint = {"groupName" : ["plateEdge"], 
              "dofConstraint" : 123456}

myProblem.analysis["nastran"].setAnalysisVal("Constraint", ("cantilever", constraint))


# Run AIM pre-analysis
myProblem.analysis["nastran"].aimPreAnalysis()

####### Run Nastran####################
print "\n\nRunning Nastran......"  
currentDirectory = os.getcwd() # Get our current working directory 

os.chdir(myProblem.analysis["nastran"].analysisDir) # Move into test directory

os.system("nastran old=no notify=no batch=yes scr=yes sdirectory=./ " + projectName +  ".dat"); # Run Nastran via system call

os.chdir(currentDirectory) # Move back to working directory 
print "Done running Nastran!"

# Run AIM post-analysis
myProblem.analysis["nastran"].aimPostAnalysis()

# Close CAPS 
myProblem.closeCAPS()
