# Import pyCAPS class file
from pyCAPS import capsProblem

# Import os module   
try: 
    import os
except:
    print "Unable to import os module"
    raise SystemExit

# Initialize pyCAPS object - directory where the pyCAPS.so resides
myProblem = capsProblem(libDir = os.environ['ESP_ROOT'] + "/lib")

# Load CSM file 
myProblem.loadCAPS("./csmData/feaSimplePlate.csm")

# Create project name 
projectName = "NastranSingleLoadPlate"

# Load Nastran aim 
myProblem.loadAIM(aim = "nastranAIM", altName = "nastran", analysisDir = projectName, capsFidelity = "STRUCTURE")


# Set project name so a mesh file is generated 
myProblem.analysis["nastran"].setAnalysisVal("Proj_Name", projectName) 

# Set meshing parameters
myProblem.analysis["nastran"].setAnalysisVal("Edge_Point_Max", 4)

myProblem.analysis["nastran"].setAnalysisVal("No_Quad_Faces", False)

myProblem.analysis["nastran"].setAnalysisVal("Tess_Params", [.25,.01,15])

# Set analysis type
myProblem.analysis["nastran"].setAnalysisVal("Analysis_Type", "Static");

# Set materials 
madeupium    = {"materialType" : "isotropic", 
                "youngModulus" : 72.0E9 , 
                "poissonRatio": 0.33, 
                "density" : 2.8E3}

myProblem.analysis["nastran"].setAnalysisVal("Material", ("Madeupium", madeupium))

# Set properties
shell  = {"propertyType" : "Shell", 
          "membraneThickness" : 0.006, 
          "material"        : "madeupium", 
          "bendingInertiaRatio" : 1.0, # Default           
          "shearMembraneRatio"  : 5.0/6.0} # Default 

myProblem.analysis["nastran"].setAnalysisVal("Property", ("plate", shell))

# Set constraints
constraint = {"groupName" : "plateEdge", 
              "dofConstraint" : 123456}

myProblem.analysis["nastran"].setAnalysisVal("Constraint", ("edgeConstraint", constraint))

# Set load
load = {"groupName" : "plate", 
        "loadType" : "Pressure", 
        "pressureForce" : 2.e6}

# Set loads    
myProblem.analysis["nastran"].setAnalysisVal("Load", ("appliedPressure", load ))

# Set analysis  
# No analysis case information needs to be set for a single static load case

# Run AIM pre-analysis
myProblem.analysis["nastran"].aimPreAnalysis()

####### Run Nastran ####################
print "\n\nRunning Nastran......"  
currentDirectory = os.getcwd() # Get our current working directory 

os.chdir(myProblem.analysis["nastran"].analysisDir) # Move into test directory

os.system("nastran old=no notify=no batch=yes scr=yes sdirectory=./ " + projectName +  ".dat"); # Run Nastran via system call

os.chdir(currentDirectory) # Move back to working directory 
print "Done running Nastran!"
######################################

# Run AIM post-analysis
myProblem.analysis["nastran"].aimPostAnalysis()

# Close CAPS 
myProblem.closeCAPS()
