# Import pyCAPS class file
from pyCAPS import capsProblem

# Import os module   
try: 
    import os
except:
    print "Unable to import os module"
    raise SystemExit

from parallel_computation import parallel_computation as su2Run

#NREL's S809 Airfoil coordinates 
#          x          y
S809 = [1.000000,  0.000000,
        0.996203,  0.000487,
        0.985190,  0.002373,
        0.967844,  0.005960,
        0.945073,  0.011024,
        0.917488,  0.017033,
        0.885293,  0.023458,
        0.848455,  0.030280,
        0.807470,  0.037766,
        0.763042,  0.045974,
        0.715952,  0.054872,
        0.667064,  0.064353,
        0.617331,  0.074214,
        0.567830,  0.084095,
        0.519832,  0.093268,
        0.474243,  0.099392,
        0.428461,  0.101760,
        0.382612,  0.101840,
        0.337260,  0.100070,
        0.292970,  0.096703,
        0.250247,  0.091908,
        0.209576,  0.085851,
        0.171409,  0.078687,
        0.136174,  0.070580,
        0.104263,  0.061697,
        0.076035,  0.052224,
        0.051823,  0.042352,
        0.031910,  0.032299,
        0.016590,  0.022290,
        0.006026,  0.012615,
        0.000658,  0.003723,
        0.000204,  0.001942,
        0.000000, -0.000020,
        0.000213, -0.001794,
        0.001045, -0.003477,
        0.001208, -0.003724,
        0.002398, -0.005266,
        0.009313, -0.011499,
        0.023230, -0.020399,
        0.042320, -0.030269,
        0.065877, -0.040821,
        0.093426, -0.051923,
        0.124111, -0.063082,
        0.157653, -0.073730,
        0.193738, -0.083567,
        0.231914, -0.092442,
        0.271438, -0.099905,
        0.311968, -0.105281,
        0.353370, -0.108181,
        0.395329, -0.108011,
        0.438273, -0.104552,
        0.481920, -0.097347,
        0.527928, -0.086571,
        0.576211, -0.073979,
        0.626092, -0.060644,
        0.676744, -0.047441,
        0.727211, -0.035100,
        0.776432, -0.024204,
        0.823285, -0.015163,
        0.866630, -0.008204,
        0.905365, -0.003363,
        0.938474, -0.000487,
        0.965086,  0.000743,
        0.984478,  0.000775,
        0.996141,  0.000290,
        1.000000,  0.000000]

# Shape design variable is 1x200 need to fill in the rest of the array with "unused values"
for i in xrange(0, 200-len(S809)):
    S809.append(1.0E6)
    
# Initialize pyCAPS object - directory where the pyCAPS.so resides
myProblem = capsProblem(libDir = os.environ['ESP_ROOT'] + "/lib")

# Load CSM file 
myProblem.loadCAPS("./csmData/cfd2DArbShape.csm")

# Create working directory variable 
workDir = "SU2AFLR4ArbShapeAnalysisTest"

# Change a design parameter - area in the geometry 
myProblem.geometry.setGeometryVal("xy", S809)

print "Saving geometry"
myProblem.geometry.saveGeometry("GenericShape", directory = workDir, extension = "egads")

# Set fidelity - could also be set during .loadAIM
myProblem.capsFidelity = "CFD"

# Load aflr4 aim 
myProblem.loadAIM(aim = "aflr4AIM", analysisDir = workDir)

myProblem.analysis["aflr4AIM"].setAnalysisVal("Mesh_Sizing", [("Airfoil"   , {"numEdgePoints" : 400}),
                                                              ("TunnelWall", {"numEdgePoints" : 50}),
                                                              ("InFlow",     {"numEdgePoints" : 25}), 
                                                              ("OutFlow",    {"numEdgePoints" : 25}), 
                                                              ("2DSlice",    {"tessParams" : [0.50, .01, 45]})])

# Make quad/tri instead of just quads  
myProblem.analysis["aflr4AIM"].setAnalysisVal("Mesh_Gen_Input_String", "mquad=1 mpp=3")

# Run AIM pre-analysis
myProblem.analysis["aflr4AIM"].aimPreAnalysis()

# NO analysis is needed - aflr4 was already ran dureing preAnalysis

# Run AIM post-analysis
myProblem.analysis["aflr4AIM"].aimPostAnalysis()

# Load SU2 aim - child of aflr4 AIM  
myProblem.loadAIM(aim = "su2AIM", 
                  altName = "su2", 
                  analysisDir = workDir, parents = ["aflr4AIM"])

projectName = "pyCAPS_su2_aflr4"
# Set project name
myProblem.analysis["su2"].setAnalysisVal("Proj_Name", projectName)

myProblem.analysis["su2"].setAnalysisVal("Mesh_ASCII_Flag", True)
  
# Set AoA number 
myProblem.analysis["su2"].setAnalysisVal("Alpha", 0.0) 

# Set equation type
myProblem.analysis["su2"].setAnalysisVal("Eqn","COMPRESSIBLE")

# Set number of iterations
myProblem.analysis["su2"].setAnalysisVal("Num_Iter",1000)

# Set CFL number schedule
myProblem.analysis["su2"].setAnalysisVal("CFL_Schedule",[1, 40])

# Set read restart option
myProblem.analysis["su2"].setAnalysisVal("Restart_Read","off")

# Set CFL number iteration schedule
myProblem.analysis["su2"].setAnalysisVal("CFL_Schedule_Iter", [1, 200])
 
# Set overwrite *.cfg file if not linking to Python library
myProblem.analysis["su2"].setAnalysisVal("Overwrite_CFG", True)

# Set the aim to 2D mode 
myProblem.analysis["su2"].setAnalysisVal("Two_Dimensional", True)

# Set Mach number 
myProblem.analysis["su2"].setAnalysisVal("Mach", 0.4)

# Set boundary conditions
inviscidBC = {"bcType" : "Inviscid", "wallTemperature" : 1}

backPressureBC = {"bcType" : "SubsonicOutflow", 
                  "staticPressure" : 101300.0}

inflow = {"bcType" : "SubsonicInflow", 
          "totalPressure" : 102010.0, 
          "totalTemperature" : 288.6,
          "uVelocity" : 1.0,
          "vVelocity" : 0.0,
          "wVelocity" : 0.0}

myProblem.analysis["su2"].setAnalysisVal("Boundary_Condition", [("Airfoil"   , inviscidBC),
                                                                ("TunnelWall", inviscidBC),
                                                                ("InFlow", inflow), 
                                                                ("OutFlow",backPressureBC)])
 
# Run AIM pre-analysis
myProblem.analysis["su2"].aimPreAnalysis()

####### Run SU2 ####################
print "\n\nRunning SU2......"  
currentDirectory = os.getcwd() # Get our current working directory 

os.chdir(myProblem.analysis["su2"].analysisDir) # Move into test directory

su2Run(projectName + ".cfg", 4); # Run SU2 
   
os.chdir(currentDirectory) # Move back to top directory 
#######################################

# Run AIM post-analysis
myProblem.analysis["su2"].aimPostAnalysis()

# Get force results
print "Total Force - Pressure + Viscous"
# Get Lift and Drag coefficients
print "Cl = " , myProblem.analysis["su2"].getAnalysisOutVal("CLtot"), \
      "Cd = " , myProblem.analysis["su2"].getAnalysisOutVal("CDtot")

# Close CAPS 
myProblem.closeCAPS()
