# Import pyCAPS class file
from pyCAPS import capsProblem

# Import os module   
try: 
    import os
except:
    print "Unable to import os module"
    raise SystemExit

from parallel_computation import parallel_computation as su2Run

# Initialize pyCAPS object - directory where the pyCAPS.so resides
myProblem = capsProblem(libDir = os.environ['ESP_ROOT'] + "/lib")

# Load CSM file 
myProblem.loadCAPS("./csmData/cfdMultiBody.csm")

# Set fidelity - could also be set during .loadAIM
myProblem.capsFidelity = "CFD"

# Change a design parameter - area in the geometry 
myProblem.geometry.setGeometryVal("area", 50)

# Load Tetgen aim 
myProblem.loadAIM(aim = "tetgenAIM", analysisDir= ".")

# Set new EGADS body tessellation parameters
myProblem.analysis["tetgenAIM"].setAnalysisVal("Tess_Params", [.05, 0.01, 20.0])

# Preserve surface mesh while messhing
myProblem.analysis["tetgenAIM"].setAnalysisVal("Preserve_Surf_Mesh", True)

# Get analysis info. 
myProblem.analysis["tetgenAIM"].getAnalysisInfo()

# Run AIM pre-analysis
myProblem.analysis["tetgenAIM"].aimPreAnalysis()

# NO analysis is needed - Tetgen was already ran dureing preAnalysis

# Run AIM post-analysis
myProblem.analysis["tetgenAIM"].aimPostAnalysis()

# Get analysis info. 
myProblem.analysis["tetgenAIM"].getAnalysisInfo()

# Load SU2 aim - child of Tetgen AIM  
myProblem.loadAIM(aim = "su2AIM", 
                  altName = "su2", 
                  analysisDir = "SU2TetgenAnalysisTest", parents = ["tetgenAIM"])

projectName = "pyCAPS_SU2_Tetgen"
# Set project name
myProblem.analysis["su2"].setAnalysisVal("Proj_Name", projectName)

myProblem.analysis["su2"].setAnalysisVal("Mesh_ASCII_Flag", False)
  
# Set AoA number 
myProblem.analysis["su2"].setAnalysisVal("Alpha", 1.0) 

# Set Mach number 
myProblem.analysis["su2"].setAnalysisVal("Mach", 0.5901)

# Set equation type
myProblem.analysis["su2"].setAnalysisVal("Eqn","COMPRESSIBLE")

# Set number of iterations
myProblem.analysis["su2"].setAnalysisVal("Num_Iter",10)

# Set CFL number schedule
myProblem.analysis["su2"].setAnalysisVal("CFL_Schedule",[0.5, 3.0])

# Set read restart option
myProblem.analysis["su2"].setAnalysisVal("Restart_Read","off")

# Set overwrite fun3d.nml if not linking to Python library
myProblem.analysis["su2"].setAnalysisVal("Overwrite_CFG", True)

# Set boundary conditions
inviscidBC1 = {"bcType" : "Inviscid"}
inviscidBC2 = {"bcType" : "Inviscid"}
myProblem.analysis["su2"].setAnalysisVal("Boundary_Condition", [("Wing1", inviscidBC1),
                                                                ("Wing2", inviscidBC2),
                                                                ("Farfield","farfield")])
 
# Run AIM pre-analysis
myProblem.analysis["su2"].aimPreAnalysis()

####### Run SU2 ####################
print "\n\nRunning SU2......"  
currentDirectory = os.getcwd() # Get our current working directory 

os.chdir(myProblem.analysis["su2"].analysisDir) # Move into test directory

numberProc = 4;
su2Run(projectName + ".cfg", numberProc) # Run SU2

os.chdir(currentDirectory) # Move back to top directory 
#######################################

# Run AIM post-analysis
myProblem.analysis["su2"].aimPostAnalysis()

# Get force results
print "Total Force - Pressure + Viscous"
# Get Lift and Drag coefficients
print "Cl = " , myProblem.analysis["su2"].getAnalysisOutVal("CLtot"), \
      "Cd = " , myProblem.analysis["su2"].getAnalysisOutVal("CDtot")

# Get Cmx, Cmy, and Cmz coefficients 
print "Cmx = " , myProblem.analysis["su2"].getAnalysisOutVal("CMXtot"), \
      "Cmy = " , myProblem.analysis["su2"].getAnalysisOutVal("CMYtot"), \
      "Cmz = " , myProblem.analysis["su2"].getAnalysisOutVal("CMZtot")
    
# Get Cx, Cy, Cz coefficients 
print "Cx = " , myProblem.analysis["su2"].getAnalysisOutVal("CXtot"), \
      "Cy = " , myProblem.analysis["su2"].getAnalysisOutVal("CYtot"), \
      "Cz = " , myProblem.analysis["su2"].getAnalysisOutVal("CZtot")

# Close CAPS 
myProblem.closeCAPS()
